==> moagg_0.17.orig.tar.gz: moagg-0.17/libMoaggObjects/include/SAMBattery.h <==
#ifndef SAMBATTERY_H
#define SAMBATTERY_H

#include "Tools.h"
#include "StaticDecorationBase.h"
#include "Missile.h"


//----------------------------------------------------------------------------
class SAMBatterySurfaces : public OrientatingSurfacesBase
{
    SURFACES_SINGLETON_OBJECT(SAMBatterySurfaces);
};


//----------------------------------------------------------------------------
class SAMBatteryBase : public StaticDecorationBase,
                       public OrientatingDecorationBase
{
  public:
    //------------------------------------------------------------------------
    class InitializationData
        : public StaticDecorationBase::InitializationData,
          public OrientatingDecorationBase::InitializationData
    {
      public:
        InitializationData();
        InitializationData(const XMLNode *samNode);
        ~InitializationData();

        unsigned delay;
        unsigned missileFuel;
        Missile::EnumHeadingStrategy missileHeadingStrategy;
        Missile::EnumWarheadStrategy missileWarheadStrategy;
    };


    //------------------------------------------------------------------------
    virtual ~SAMBatteryBase();

    //------------------------------------------------------------------------
    void update();

    //------------------------------------------------------------------------
    void updateSurface();

    //------------------------------------------------------------------------
    static SAMBatteryBase *create(const InitializationData &init);
    static SAMBatteryBase *create(const XMLNode *samNode);

  protected:
    //------------------------------------------------------------------------
    SAMBatteryBase(const InitializationData &init);

    //------------------------------------------------------------------------
    virtual Missile *createMissile() const = 0;
    Missile *do_createMissile(Sint16 angle) const;

    //------------------------------------------------------------------------
    unsigned m_frameDelay;
    unsigned m_frameCounter;

  private:
    /// Helper method to set the missile's initial position.
    virtual void do_setInitialMissilePosition(Missile *m) const = 0;

    //------------------------------------------------------------------------
    DECLARE_OBJECT_VISITOR_API();


    /// The initial fuel to use for shooted missiles.
    unsigned m_missileFuel;

    /// The heading strategy to use for shooted missiles.
    Missile::EnumHeadingStrategy m_missileHeadingStrategy;

    /// The warhead strategy to use for shooted missiles.
    Missile::EnumWarheadStrategy m_missileWarheadStrategy;
};


//----------------------------------------------------------------------------
class TopSAMBattery : public SAMBatteryBase
{
  public:
    TopSAMBattery(const InitializationData &init);
    ~TopSAMBattery();

  protected:
    virtual Missile *createMissile() const;

  private:
    void do_setInitialMissilePosition(Missile *m) const;
};

//----------------------------------------------------------------------------
class BottomSAMBattery : public SAMBatteryBase
{
  public:
    BottomSAMBattery(const InitializationData &init);
    ~BottomSAMBattery();

  protected:
    virtual Missile *createMissile() const;

  private:
    void do_setInitialMissilePosition(Missile *m) const;
};

//----------------------------------------------------------------------------
class LeftSAMBattery : public SAMBatteryBase
{
  public:
    LeftSAMBattery(const InitializationData &init);
    ~LeftSAMBattery();

  protected:
    virtual Missile *createMissile() const;

  private:
    void do_setInitialMissilePosition(Missile *m) const;
};

//----------------------------------------------------------------------------
class RightSAMBattery : public SAMBatteryBase
{
  public:
    RightSAMBattery(const InitializationData &init);
    ~RightSAMBattery();

  protected:
    virtual Missile *createMissile() const;

  private:
    void do_setInitialMissilePosition(Missile *m) const;
};

#endif //SAMBATTERY_H

==> moagg_0.18.orig.tar.gz: moagg-0.18/libMoaggObjects/include/SAMBattery.h <==
#ifndef SAMBATTERY_H
#define SAMBATTERY_H

#include "Tools.h"
#include "StaticDecorationBase.h"
#include "Missile.h"


//----------------------------------------------------------------------------
class SAMBatterySurfaces : public OrientatingSurfacesBase
{
    SURFACES_SINGLETON_OBJECT(SAMBatterySurfaces);
};


//----------------------------------------------------------------------------
class SAMBatteryBase : public StaticDecorationBase,
                       public OrientatingDecorationBase
{
  public:
    //------------------------------------------------------------------------
    class InitializationData
        : public StaticDecorationBase::InitializationData,
          public OrientatingDecorationBase::InitializationData
    {
      public:
        InitializationData();
        InitializationData(const XMLNode *samNode);
        ~InitializationData();

        unsigned hitPoints;
        unsigned delay;
        unsigned missileFuel;
        Missile::EnumHeadingStrategy missileHeadingStrategy;
        Missile::EnumWarheadStrategy missileWarheadStrategy;
    };


    //------------------------------------------------------------------------
    virtual ~SAMBatteryBase();

    //------------------------------------------------------------------------
    /**
     * Called for a projectile collision to determine,
     * if the SAM's hitpoints are reached.
     */
    inline bool decAndTestHitPoints()
    {
        return --m_hitPoints == 0;
    }

    //------------------------------------------------------------------------
    void update();

    //------------------------------------------------------------------------
    void updateSurface();

    //------------------------------------------------------------------------
    static SAMBatteryBase *create(const InitializationData &init);
    static SAMBatteryBase *create(const XMLNode *samNode);

  protected:
    //------------------------------------------------------------------------
    SAMBatteryBase(const InitializationData &init);

    //------------------------------------------------------------------------
    virtual Missile *createMissile() const = 0;
    Missile *do_createMissile(Sint16 angle) const;

    //------------------------------------------------------------------------
    unsigned m_hitPoints;

    unsigned m_frameDelay;
    unsigned m_frameCounter;

  private:
    /// Helper method to set the missile's initial position.
    virtual void do_setInitialMissilePosition(Missile *m) const = 0;

    //------------------------------------------------------------------------
    DECLARE_OBJECT_VISITOR_API();


    /// The initial fuel to use for shooted missiles.
    unsigned m_missileFuel;

    /// The heading strategy to use for shooted missiles.
    Missile::EnumHeadingStrategy m_missileHeadingStrategy;

    /// The warhead strategy to use for shooted missiles.
    Missile::EnumWarheadStrategy m_missileWarheadStrategy;
};


//----------------------------------------------------------------------------
class TopSAMBattery : public SAMBatteryBase
{
  public:
    TopSAMBattery(const InitializationData &init);
    ~TopSAMBattery();

  protected:
    virtual Missile *createMissile() const;

  private:
    void do_setInitialMissilePosition(Missile *m) const;
};

//----------------------------------------------------------------------------
class BottomSAMBattery : public SAMBatteryBase
{
  public:
    BottomSAMBattery(const InitializationData &init);
    ~BottomSAMBattery();

  protected:
    virtual Missile *createMissile() const;

  private:
    void do_setInitialMissilePosition(Missile *m) const;
};

//----------------------------------------------------------------------------
class LeftSAMBattery : public SAMBatteryBase
{
  public:
    LeftSAMBattery(const InitializationData &init);
    ~LeftSAMBattery();

  protected:
    virtual Missile *createMissile() const;

  private:
    void do_setInitialMissilePosition(Missile *m) const;
};

//----------------------------------------------------------------------------
class RightSAMBattery : public SAMBatteryBase
{
  public:
    RightSAMBattery(const InitializationData &init);
    ~RightSAMBattery();

  protected:
    virtual Missile *createMissile() const;

  private:
    void do_setInitialMissilePosition(Missile *m) const;
};

#endif //SAMBATTERY_H
